@@ -1,11 +1,10 @@
 package CGI::Application::Plugin::FillInForm;
+use base 'Exporter';
 use strict;
-require Exporter;
-use vars (qw/@ISA @EXPORT_OK $VERSION/);
+use warnings;
 
-$VERSION = '1.13';
-@ISA = qw(Exporter);
-@EXPORT_OK = qw(fill_form);
+our $VERSION = '1.15';
+our @EXPORT_OK = qw(fill_form);
 use Carp;
 
 =head1 NAME
@@ -39,7 +38,7 @@ convenient.
  my $filled_html = $self->fill_form($html);
 
  # extra fields will be passed on through:
- my $filled_html = $self->fill_form($html, undef, fill_passwords => 0 );
+ my $filled_html = $self->fill_form($html, undef, fill_password => 0 );
 
 This method provides an easier syntax for calling HTML::FillInForm, and an
 intelligent default of using $self->query() as the default data source.
@@ -48,7 +47,7 @@ By default, the mode param (usually 'rm') of every data source will be
 ignored. This prevents accidently clobbering your run mode for the next
 page, which may be stored in a hidden field.
 
-B<$html> must be a scalarref.
+B<$html> must be a scalarref, or a reference to a scalarref.
 B<$filled_html> will be a reference to a string.
 
 Because this method only loads HTML::FillInForm if it's needed, it should be
@@ -64,6 +63,9 @@ sub fill_form {
     my $data = shift;
     my %extra_params = @_;
 
+    if (ref $html eq 'REF' and ref $$html eq 'SCALAR') {
+        $html = $$html;
+    }
     die "html must be a scalarref!" unless (ref $html eq 'SCALAR');
 
     my %params = (
@@ -75,7 +77,7 @@ sub fill_form {
 
         $data = [$data] unless ref $data eq 'ARRAY';
 
-        foreach my $source (@$data) {
+        for my $source (@$data) {
             if (ref $source eq 'HASH') {
                 push @fdat, $source;
             }
@@ -100,8 +102,8 @@ sub fill_form {
         if (@fdat) {
             if (@fdat > 1) {
                 my %merged;
-                foreach my $hash (@fdat) {
-                    foreach my $key (keys %$hash) {
+                for my $hash (@fdat) {
+                    for my $key (keys %$hash) {
                         $merged{$key} = $hash->{$key};
                     }
                 }
